package cn.tonyandmoney.tuon.core.config

import cn.tonyandmoney.tuon.core.properties.CustomConfigProperties
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Primary
import org.springframework.http.codec.ServerCodecConfigurer
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter
import org.springframework.web.reactive.config.ResourceHandlerRegistry
import org.springframework.web.reactive.config.WebFluxConfigurationSupport
import java.text.SimpleDateFormat

/**
 * Created by niantuo on 2018/9/20.
 * mvc 配置
 *
 */
@Configuration
@EnableConfigurationProperties(CustomConfigProperties::class)
@ConditionalOnProperty(prefix = "custom.config.enable", name = ["web"], havingValue = "true", matchIfMissing = true)
class CustomWebMvcConfiguration : WebFluxConfigurationSupport() {

    companion object {
        const val CONVERTER_NAME = "hookMappingJackson2HttpMessageConverter"
    }

    override fun addResourceHandlers(registry: ResourceHandlerRegistry) {
        registry.addResourceHandler("swagger-ui.html")
                .addResourceLocations("classpath:/META-INF/resources/")


        registry.addResourceHandler("/webjars*")
                .addResourceLocations("classpath:/META-INF/resources/webjars/")
    }

    override fun configureHttpMessageCodecs(configurer: ServerCodecConfigurer) {
        configurer.registerDefaults(true)
    }


    @Bean(CONVERTER_NAME)
    fun getConverter(): MappingJackson2HttpMessageConverter {
        return MappingJackson2HttpMessageConverter().apply {
            this.objectMapper = builder().build()
        }
    }

    @Bean
    fun xmlConverter():MappingJackson2HttpMessageConverter{
        return MappingJackson2HttpMessageConverter().apply {
            this.objectMapper = Jackson2ObjectMapperBuilder.xml().build()
        }

    }

    @Bean("xmlObjectMapper")
    fun xmlObjectMapper():ObjectMapper{
        return Jackson2ObjectMapperBuilder.xml().build()
    }

    @Bean
    @Primary
    fun builder(): Jackson2ObjectMapperBuilder {
        return Jackson2ObjectMapperBuilder().apply {
            serializerByType(java.lang.Long::class.java, ToStringSerializer.instance)
            serializerByType(java.lang.Long.TYPE, ToStringSerializer.instance)
            // date
            dateFormat(SimpleDateFormat("yyyy-MM-dd HH:mm:ss"))
            timeZone("GMT+8")
        }
    }
}