package cn.tonyandmoney.tuon.qywx.controller


import cn.tonyandmoney.tuon.core.error.BaseResp
import cn.tonyandmoney.tuon.core.error.OpException
import cn.tonyandmoney.tuon.core.error.SessionNoUserException
import cn.tonyandmoney.tuon.core.session.SessionUtils
import cn.tonyandmoney.tuon.core.user.IUser
import cn.tonyandmoney.tuon.qywx.QywxProperties
import cn.tonyandmoney.tuon.qywx.WxErrorCode
import cn.tonyandmoney.tuon.qywx.aes.WXBizMsgCrypt
import cn.tonyandmoney.tuon.qywx.bean.WxMsg
import cn.tonyandmoney.tuon.qywx.bean.WxUser
import cn.tonyandmoney.tuon.qywx.service.IQywxService
import com.fasterxml.jackson.databind.ObjectMapper
import org.slf4j.LoggerFactory
import org.slf4j.MarkerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.core.io.buffer.DataBufferUtils
import org.springframework.http.server.reactive.ServerHttpRequest
import org.springframework.web.bind.annotation.*
import org.springframework.web.server.ServerWebExchange
import reactor.core.publisher.Mono

@RestController
@RequestMapping("/qywx")
class QywxController {

    companion object {
        private val logger = LoggerFactory.getLogger(QywxController::class.java.simpleName)
        private val msgMarker = MarkerFactory.getMarker("WxMsg")
    }

    @Autowired
    private lateinit var qywxService: IQywxService
    @Autowired
    private lateinit var properties: QywxProperties
    @Autowired
    private lateinit var wxBizMsgCrypt: WXBizMsgCrypt
    @Autowired
    @Qualifier("xmlObjectMapper")
    private lateinit var objectMapper: ObjectMapper

    @GetMapping("/user")
    fun getUserInfo(@RequestParam("code", required = false) code: String?,
                    @RequestParam("agentId") agentId: String,
                    exchange: ServerWebExchange): Mono<BaseResp<IUser>> {

        return SessionUtils.getUser<WxUser>(exchange)
                .onErrorResume(SessionNoUserException::class.java) {
                    if (code.isNullOrBlank()) {
                        Mono.error(OpException(WxErrorCode.ERR_CODE))
                    } else {
                        qywxService.getUserInfo(agentId, code)
                                .flatMap { user -> qywxService.getUserById(agentId, user.id) }
                                .flatMap { info -> SessionUtils.setUser(exchange, info) }
                    }
                }
                .flatMap { info -> Mono.just(BaseResp.ok(info)) }
                .onErrorResume {
                    if (it is OpException) {
                        Mono.just(it.toResp())
                    } else {
                        Mono.just(BaseResp.error(it))
                    }
                }


    }

    /**
     * 接收消息
     */
    @PostMapping("/config/message")
    fun configUserMessage(@RequestParam("msg_signature") signature: String,
                          @RequestParam("timestamp") timestamp: String,
                          @RequestParam("nonce") nonce: String,
                          request: ServerHttpRequest): Mono<String> {

        return DataBufferUtils.join(request.body)
                .flatMap { dataBuffer ->
                    Mono.create<String> {
                        val bytes = ByteArray(dataBuffer.readableByteCount())
                        dataBuffer.read(bytes)
                        logger.info(msgMarker, "content:{}", String(bytes))
                        val wxMsg = objectMapper.readValue<WxMsg>(bytes, WxMsg::class.java)
                        val content = wxBizMsgCrypt.decryptMsg(signature, timestamp, nonce, wxMsg.encrypt)
                        val resp = ""
                        if ("值班" == content) {

                        } else {

                        }
                        Mono.just(resp)
                    }
                }

    }

    /**
     * 验证URL
     */
    @GetMapping("/config/message")
    fun verifySignature(@RequestParam("msg_signature") signature: String,
                        @RequestParam("timestamp") timestamp: String,
                        @RequestParam("nonce") nonce: String,
                        @RequestParam("echostr") echostr: String): Mono<String> {
        return Mono.create {
            logger.info(msgMarker,"signature:{},timestamp:{},nonce:{},echostr:{}",signature,timestamp,nonce,echostr)
            val utils = WXBizMsgCrypt(properties.token, properties.encodingAESKey, properties.corpid)
            it.success(utils.VerifyURL(signature, timestamp, nonce, echostr))
        }
    }

}
