package cn.gygxzc.envir.sys.service

import cn.gygxzc.cloud.tina.auth.annotation.ResourceApi
import cn.gygxzc.cloud.tina.auth.enums.ResourceLevel
import cn.gygxzc.tina.cloud.jwt.session.bean.User
import org.springframework.cloud.openfeign.FeignClient
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestMethod

/**
 * Created by niantuo on 2018/12/17.
 * 通过feign 调用获取用户信心。
 */
@FeignClient("sys")
interface IFUserService {


    /**
     * 根据用户的id获取用户信息
     */
    @RequestMapping(method = [RequestMethod.POST], value = ["/feign/user"])
    fun getUserInfoByIds(@RequestBody ids: List<Long>): List<User>

    /**
     * 根据用户的id获取用户信息
     */
    @RequestMapping("/feign/user/{id}", method = [RequestMethod.GET])
    fun getUserInfo(@PathVariable("id") id: Long): User?

    /**
     * 根据用户的账号获取用户信息
     */
    @RequestMapping("/feign/user/info/{account}", method = [RequestMethod.GET])
    fun getUserByAccount(@PathVariable("account") account: String): User?

    /**
     * 根据用户的id获取用户的账号信息
     */
    @RequestMapping("/feign/user/account/{id}", method = [RequestMethod.GET])
    fun getAccountById(@PathVariable("id") id: Long): String?

    /**
     * 根据用户id获取用户的账号信息...
     * 这个不知道有什么用...
     */
    @RequestMapping("/feign/user/accounts", method = [RequestMethod.POST])
    fun getAccountByIds(@RequestBody ids: List<Long>): List<String>


    /**
     * 根据用户的id修改用户信息，并不是全部信息。
     */
    @RequestMapping("/feign/user/{id}", method = [RequestMethod.PUT])
    fun updateUserInfo(@PathVariable("id") id: Long, @RequestBody user: User)

    /**
     * 根据用户账户批量获取用户信息
     */
    @RequestMapping("/feign/user/info", method = [RequestMethod.POST])
    @ResourceApi(name = "", level = ResourceLevel.AUTHC)
    fun getUserInfoByAccounts(@RequestBody accounts: List<String>): List<User>

}
